#include <stdio.h>
#include <math.h>

/* The bisection method is meant to compute the zeros of 
   a continuous function f in an interval [a,b] in which 
   f(a)*f(b) < 0. So, in order to compute the square root 
   of a number, we need to implement a function whose 
   zeros correspond to the root of a parameter a: this
   function is root_square_delta */
double root_square_delta(const double a, const double x)
{
   return x*x-a;
}

/* This is a general bisection method for functions whose 
   signature is f(const double a, const double x) where 
   a is a parameter (e.g., root_square_delta) */
double get_delta_root(double (*f)(const double, const double),
                      double a,
		      double lb, 
		      double rb,
		      const double delta)
{
   if (lb==rb) return lb;

   double m=(lb+rb)/2;
   double f_m=f(a,m);

   if (fabs(f_m)<delta) return m;

   if (f(a,lb)*f_m < 0) {
      return get_delta_root(f, a, lb, m, delta);
   } else {
      return get_delta_root(f, a, m, rb, delta);
   }
}


double square_root(const double a, const double delta)
{
   /* The inteval to be investigated to compute the 
      square root of a value depends on the value itself:
      if the value is smaller than 1, then we must 
      consider the interval [0,1];
      if the value is greater than or equal to 1, then, we must 
      take care of the interval [1, a] */

   if (a<1.0) {
	return get_delta_root(root_square_delta, a, 0.0, 1.0, delta);
   }

   if (a>1.0) {
	return get_delta_root(root_square_delta, a, 1.0, a, delta);
   }

   return 1.0;
}

int main(int argc, char *argv[])
{
   if (argc<2) {
     fprintf(stderr, "Syntax error: expected one parameter\n%s <floating point number>\n",          
             argv[0]);
     return 1;
   }
	
   double a;
   sscanf(argv[1], "%lf", &a);   

   double x = square_root(a, 1e-10);

   printf("%lf is the root of f(x)=x*x-%lf:\n%lf^2=%lf\n%lf^2-%lf^2=%.3e\n", 
          x, a, x, x*x, x, a, x*x-a);

   return 0;
}
